/**
 * Internal dependencies
 */
import {
	HAPPYCHAT_OPEN,
	HAPPYCHAT_MINIMIZING,
	HAPPYCHAT_BLUR,
	HAPPYCHAT_FOCUS,
	HAPPYCHAT_IO_SEND_MESSAGE_MESSAGE,
	HAPPYCHAT_SET_CURRENT_MESSAGE,
} from 'calypso/state/action-types';
import { combineReducers, withSchemaValidation, withPersistence } from 'calypso/state/utils';

/**
 * Tracks the current message the user has typed into the happychat client
 *
 * @param  {object} state  Current state
 * @param  {object} action Action payload
 * @returns {object}        Updated state
 *
 */
export const currentMessage = ( state = '', action ) => {
	switch ( action.type ) {
		case HAPPYCHAT_IO_SEND_MESSAGE_MESSAGE:
			return '';
		case HAPPYCHAT_SET_CURRENT_MESSAGE:
			return action.message;
	}
	return state;
};

const lostFocusAtSchema = { type: 'number' };
/**
 * Tracks the last time Happychat had focus. This lets us determine things like
 * whether the user has unread messages. A numerical value is the timestamp where focus
 * was lost, and `null` means HC currently has focus.
 *
 * @param {object} state Current state
 * @param {object} action Action payload
 * @returns {object}        Updated state
 */
export const lostFocusAt = withSchemaValidation(
	lostFocusAtSchema,
	withPersistence(
		( state = null, action ) => {
			switch ( action.type ) {
				case HAPPYCHAT_BLUR:
					return Date.now();
				case HAPPYCHAT_FOCUS:
					return null;
			}
			return state;
		},
		{
			serialize: ( state ) => {
				// If there's already a timestamp set, use that. Otherwise treat serialization as a
				// "loss of focus" since it represents the state when the browser (and HC) closed.
				if ( state === null ) {
					return Date.now();
				}
				return state;
			},
		}
	)
);

const isOpen = ( state = false, action ) => {
	switch ( action.type ) {
		case HAPPYCHAT_OPEN:
			return !! action.isOpen;
	}
	return state;
};

/**
 * Tracks the state of the happychat minimizing process
 *
 * @param  {object} state  Current state
 * @param  {object} action Action payload
 * @returns {object}        Updated state
 *
 */
const isMinimizing = ( state = false, action ) => {
	switch ( action.type ) {
		case HAPPYCHAT_MINIMIZING:
			return action.isMinimizing ? true : false;
	}
	return state;
};

export default combineReducers( { currentMessage, isMinimizing, isOpen, lostFocusAt } );
